'' =================================================================================================
''
''   File....... jm_sampler_demo.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2017 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 15 MAR 2017
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us
                                                                 
                                                                 
con { io pins }                                                  
                                                                   
  RX1  = 31                                                     ' programming / terminal
  TX1  = 30                                                      
                                                                 
  SDA1 = 29                                                     ' eeprom / i2c
  SCL1 = 28                                                      
                                                  
                                                                 
obj                                                              
                                                                 
' main                                                          ' * master Spin cog
  time : "jm_time"                                              '   timing and delays
  io   : "jm_io_basic"                                          '   essential io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var                                                              
                                                                 
                                                                 
pub main | t, idx                                                        
                                                                 
  setup

  term.rxflush
  term.rx                                                       ' open terminal, press key

  term.tx(term#CLS)
  term.str(string("Starting in... "))
  repeat idx from 3 to 1
    term.tx("0" + idx)
    term.str(string("... "))
    time.pause(1000)
  
  term.str(string(13, "Collecting Samples", 13))                                                   

  sampler_start(100, 100)                                       ' 100 samples, 100ms/sample
  repeat while (sampler_busy)

  ' playback

  dira |= LEDS_MASK                                             ' make leds outputs in this cog

  t := cnt
  repeat
    term.str(string("Playing Samples", 13)) 
    repeat idx from 0 to 99
      outa[23..16] := samples[idx]
      waitcnt(t += constant(100 * MS_001))
                                                                   
                                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays
                                                                 
  io.start(0, 0)                                                ' clear all pins (master cog)
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal
                                                                 

con { sampler }

' When ready to deploy
' -- remove "sampler_" prefix from methods
' -- save everything from this section to end in separate file


con

  MAX_SAMPLES = 4096


var

  long  scog
  long  scsetup1
  long  scsetup2
  
  word  samples[MAX_SAMPLES]


pub sampler_start(n, ms)

'' Start sampler with scan rate of ms milliseconds
'' -- n is number of samples to take
'' -- ms is milliseconds per sample, 50..1000

  sampler_stop

  scsetup1.word[0] := @samples                                  ' storage
  scsetup1.word[1] := 1 #> n <# MAX_SAMPLES                     ' #of samples
                                           
  scsetup2 := (50 #> ms <# 1000) * (clkfreq / 1000)             ' ticks in sample time
  
  scog := cognew(@sampler, @scog) + 1                           ' point to cog id


pub sampler_stop

  if (scog)
    cogstop(scog - 1)
    
  scog := 0
  wordfill(@samples, 0, MAX_SAMPLES)                            ' clear samples array


pub sampler_busy

  return (scog > 0)


dat { sampler cog }

                        org 0

sampler                 mov     t1, par                         ' get address of scog
                        add     t1, #4                          ' skip over scog
                        rdlong  shub, t1                        ' read storage & #samples
                        mov     count, shub                     ' copy

                        shl     shub, #16                       ' extract word[0] (@samples)
                        shr     shub, #16
                        shr     count, #16                      ' extract word[1] 

                        add     t1, #4                          ' point to timing
                        rdlong  smpltix, t1

                        rdlong  ms1tix, #0                      ' read system frequency
                        shr     ms1tix, #10                     ' ms1tix / 1024 (~1ms)

                        mov     outa, #0                        ' ensure everything clear
                        mov     dira, LEDS_MASK                 ' make leds outputs
                        
                        mov     stimer, smpltix                 ' initialize samples timer
                        add     stimer, cnt
                        
get_samples             call    #read_pads                      ' get pad states
                        call    #show_pads                      ' output to leds

                        wrword  newsample, shub                 ' write new sample to hub
                        add     shub, #2                        ' advance sample pointer                     
                        waitcnt stimer, smpltix                 ' let sample window finish
                        djnz    count, #get_samples

                        
auto_shutdown           mov     t1, #0
                        wrlong  t1, par                         ' alert hub we're shutting down
                        cogid   t1                              ' do it
                        cogstop t1
                        jmp     #$


read_pads               or      outa, PADS_MASK                 ' charge pads
                        or      dira, PADS_MASK
                        mov     t1, ms1tix                      
                        shl     t1, #1                          ' ~2ms   
                        add     t1, cnt
                        waitcnt t1, #0
                        andn    dira, PADS_MASK                 ' let pads float
                        mov     t1, ms1tix
                        shl     t1, #5                          ' ~32ms
                        add     t1, cnt
                        waitcnt t1, #0
                        mov     newsample, ina                  ' read the inputs
                        xor     newsample, #$FF                 ' invert (1 means pressed)
                        and     newsample, #$FF                 ' clean-up
read_pads_ret           ret
                        

show_pads               andn    outa, LEDS_MASK                 ' clear leds
                        mov     t1, newsample                   ' make copy of pad bits
                        shl     t1, #16                         ' align with leds
                        or      outa, t1                        ' light leds with pad bits
show_pads_ret           ret  


' -------------------------------------------------------------------------------------------------

PADS_MASK               long    $FF << 0                        ' P07..P00
LEDS_MASK               long    $FF << 16                       ' P23..P16

shub                    res     1                               ' base address of hub storage  
count                   res     1                               ' # of samples to take
smpltix                 res     1                               ' ticks in sample window

ms1tix                  res     1                               ' ticks in ~1ms
stimer                  res     1                               ' samples timer

newsample               res     1                               ' sample data (up to 16 bits)

t1                      res     1
t2                      res     1

                        fit     496

                                                                 
dat { license }

  Copyright   byte      "Copyright (C) 2017 Jon McPhalen", 0                                                 

{{

  Copyright (C) 2017 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}